//======================================================================
//  flpost; main.h
//    Author: Dave Freese, W1HKJ <w1hkj@w1hkj.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the
//
//  Free Software Foundation, Inc.
//  51 Franklin Street, Fifth Floor
//  Boston, MA  02110-1301 USA.
//
// ---------------------------------------------------------------------

#ifndef FLPOST_MAIN_H
#define FLPOST_MAIN_H
/** \defgroup FLPOST */

/** \ingroup FLPOST
 * \brief Main entry point for FLPost
 * \details 
 *
 *  flpost; main<br>
 *  &nbsp;&nbsp;&nbsp;&nbsp;Author: Dave Freese, W1HKJ (w1hkj@w1hkj.com)<br>
 *
 * This program is free software; you can redistribute it and/or <br>
 * modify it under the terms of the GNU General Public License <br>
 * as published by the Free Software Foundation; either version 2 <br>
 * of the License, or any later version. <br><br>
 *
 * This program is distributed in the hope that it will be useful, <br>
 * but WITHOUT ANY WARRANTY; without even the implied warranty of <br>
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the <br>
 * GNU General Public License for more details. <br>
 *
 * You should have received a copy of the GNU General Public License<br>
 * along with this program; if not, write to the <br><br>
 *
 *  Free Software Foundation, Inc. <br>
 *  51 Franklin Street, Fifth Floor <br>
 *  Boston, MA  02110-1301 USA. <br><br>  */
#include <string>
#include <fstream>
#include <ostream>
#include <cstring>
#include <ctime>
#include <vector>

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <FL/Fl.H>
#include <FL/Enumerations.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Sys_Menu_Bar.H>
#include <FL/x.H>
#include <FL/Fl_Help_Dialog.H>
#include <FL/Fl_Menu_Item.H>
#include <FL/fl_ask.H>
#include <FL/Fl_File_Chooser.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Image.H>

#include "smtp-tls.h"

extern std::string BaseDir;
extern std::string HomeDir;
extern c_smtp_mail mail_io;

extern Fl_Double_Window *flpost_main_window;
extern Fl_Double_Window *flpost_config_window;

extern int main(int argc, char **argv);

extern int handle (int e);
extern void visit_URL(void* arg);
extern void clean_exit();
extern void exit_main(Fl_Widget *w);
extern void fatal_error(std::string sz_error);
extern int parse_args(int argc, char **argv, int& idx);
extern void close_output_dialog();
extern void about();
extern void flpost_configure();
extern void close_config();
extern void send();
extern void defaults();

extern void cb_message_to();
extern void cb_message_from();
extern void cb_message_cc();
extern void cb_message_subj();
extern void cb_message_body();
extern void cb_mail_out_url();
extern void cb_mail_out_port();
extern void cb_mail_in_url();
extern void cb_mail_in_port();
extern void cb_mail_user_name();
extern void cb_mail_user_password();
extern void set_post_master();
extern void post_server_name();
extern void clear_mail();

#if !defined(__WIN32__) && !defined(__APPLE__)
	extern void make_pixmap(Pixmap *xpm, const char **data);
#endif

#endif // FLPOST_MAIN_H
