//======================================================================
//  flpost; smtp-tls.cxx
//    Author: Dave Freese, W1HKJ <w1hkj@w1hkj.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the
//
//  Free Software Foundation, Inc.
//  51 Franklin Street, Fifth Floor
//  Boston, MA  02110-1301 USA.
//
// ---------------------------------------------------------------------

/* <DESC>
 * SMTP example using TLS
 * </DESC>
 */
#include <iostream>

#include "smtp-tls.h"
#include "debug.h"

using namespace std;

size_t c_smtp_mail::payload_source(void *ptr, size_t size, size_t nmemb, void *userp)
{
	c_smtp_mail *me = (c_smtp_mail *)userp;
	if (!me->payload.empty()) {
		size_t len = me->payload.length();
		memcpy(ptr, me->payload.c_str(), len);
		me->payload.clear();
		return len;
	}
	return 0;
}

const char * c_smtp_mail::hexstr(int d)
{
	static char hxstr[7] = "<0x  >";
	static const char hex[] = "0123456789ABCDEF";
	hxstr[3] = hex[(d & 0xF0) >> 8];
	hxstr[4] = hex[d & 0x0F];
	return hxstr;
}

std::string c_smtp_mail::dump_string(std::string d_in)
{
	std::string d_out;
	d_out.clear();
	for (size_t n = 0; n < d_in.length(); n++) {
		if (d_in[n] < 0x20 && d_in[n] != '\n') d_out.append(hexstr(d_in[n]));
		else if (d_in[n] > 0x7E) d_out.append(hexstr(d_in[n]));
		else d_out += d_in[n];
	}
	return d_out;
}

int c_smtp_mail::my_trace(
	CURL *handle,
	curl_infotype type,
	char *data,
	size_t size,
	void *userp)
{
	std::string debug_txt;

//	const char *text;
	(void)handle; /* prevent compiler warning */

	c_smtp_mail *me = (c_smtp_mail *)userp;
 
	switch (type) {
	default: /* in case a new one is introduced to shock us */
		return 0;
	case CURLINFO_TEXT:
		debug_txt.append("Info: ");
		break;
	case CURLINFO_HEADER_OUT:
		debug_txt.append("Send header: ");
		break;
	case CURLINFO_DATA_OUT:
		debug_txt.append("Send data: ");
		break;
	case CURLINFO_SSL_DATA_OUT:
		debug_txt.append("Send SSL data: ");
		break;
	case CURLINFO_HEADER_IN:
		debug_txt.append("Recv header: ");
		break;
	case CURLINFO_DATA_IN:
		debug_txt.append("Recv data: ");
		break;
	case CURLINFO_SSL_DATA_IN:
		debug_txt.append("Recv SSL data: ");
		break;
	}
	debug_txt.append(me->dump_string(data));
	LOG_INFO("%s", debug_txt.c_str());

	return 0;
}

int c_smtp_mail::send_mail(void)
{
	if (was_sent) return 0;

	if (from_.empty()) from_ = FROM;
	if (to_.empty()) to_ = TO;
//	if (cc_.empty()) cc_ = CC;
	if (subj_.empty()) subj_ = SUBJ;
	if (body_.empty()) body_ = BODY;
	if (username_.empty()) username_ = USERNAME;
	if (password_.empty()) password_ = PASSWORD;
	if (server_url_.empty()) server_url_ = SMTP_SERVER;
	if (server_port_.empty()) server_port_ = SMTP_PORT;

	time_t t = time(NULL);
	std::string stime = ctime(&t);

	payload.assign("Date: ").append(ctime(&t));
	payload.append("To: ").append(to()).append("\r\n");
	payload.append("From: ").append(from()).append("\r\n");
	payload.append("Cc: ").append(cc()).append("\r\n");
	payload.append("Subject: ").append(subj()).append("\r\n");
	payload.append("\r\n");
	payload.append(body()).append("\r\n");

	if (!curl) curl = curl_easy_init();
	if(curl) {

		curl_easy_setopt(curl, CURLOPT_DEBUGFUNCTION, my_trace);
		curl_easy_setopt(curl, CURLOPT_DEBUGDATA, this);
 
/* the DEBUGFUNCTION has no effect until we enable VERBOSE */
		curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
 
/* Set username and password */
		curl_easy_setopt(curl, CURLOPT_USERNAME, username_.c_str());
		curl_easy_setopt(curl, CURLOPT_PASSWORD, password_.c_str());

/* This is the URL for your mailserver. */
		curl_easy_setopt(
			curl,
			CURLOPT_URL,
			string(server_url_).append(":").append(server_port_).c_str());

/* Start with a plain text connection, and upgrade to Transport Layer 
 * Security (TLS) using the STARTTLS command. 
 * Be careful of using CURLUSESSL_TRY here, if TLS upgrade fails, the
 * transfer will continue anyway - see the security discussion in the
 * libcurl tutorial for more details. */
	 
		curl_easy_setopt(curl, CURLOPT_USE_SSL, (long)CURLUSESSL_ALL);

/* If your server doesn't have a valid certificate, then you can disable
 * part of the Transport Layer Security protection by setting the
 * CURLOPT_SSL_VERIFYPEER and CURLOPT_SSL_VERIFYHOST options to 0 (false).
 *   curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
 *   curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
 * That is, in general, a bad idea. It is still better than sending your
 * authentication details in plain text though.  Instead, you should get
 * the issuer certificate (or the host certificate if the certificate is
 * self-signed) and add it to the set of certificates that are known to
 * libcurl using CURLOPT_CAINFO and/or CURLOPT_CAPATH. See docs/SSLCERTS
 * for more information. */

		curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
		curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0L);
//    curl_easy_setopt(curl, CURLOPT_CAINFO, "/path/to/certificate.pem");

		/* Note that this option isn't strictly required, omitting it will result
		 * in libcurl sending the MAIL FROM command with empty sender data. All
		 * autoresponses should have an empty reverse-path, and should be directed
		 * to the address in the reverse-path which triggered them. Otherwise,
		 * they could cause an endless loop. See RFC 5321 Section 4.5.5 for more
		 * details.
		 */
		curl_easy_setopt(curl, CURLOPT_MAIL_FROM, FROM);

/* Add two recipients, in this particular case they correspond to the
 * To: and Cc: addressees in the header, but they could be any kind of
 * recipient. */
		recipients = curl_slist_append(recipients, to_.c_str());
		recipients = curl_slist_append(recipients, cc_.c_str());
		curl_easy_setopt(curl, CURLOPT_MAIL_RCPT, recipients);

/* We're using a callback function to specify the payload (the headers and
 * body of the message). You could just use the CURLOPT_READDATA option to
 * specify a FILE pointer to read from. */
		curl_easy_setopt(curl, CURLOPT_READFUNCTION, payload_source);
		curl_easy_setopt(curl, CURLOPT_READDATA, this);

		curl_easy_setopt(curl, CURLOPT_UPLOAD, 1L);

/* Send the message */
		res = curl_easy_perform(curl);

/* Check for errors */
		if(res != CURLE_OK)
			fprintf(stderr, "curl_easy_perform() failed: %s\n",
							curl_easy_strerror(res));

/* Free the list of recipients */
		curl_slist_free_all(recipients);

/* Always cleanup */
		curl_easy_cleanup(curl);
	}

	was_sent = true;

	return res;
}
