//======================================================================
//  flpost; main
//    Author: Dave Freese, W1HKJ <w1hkj@w1hkj.com>
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the
//
//  Free Software Foundation, Inc.
//  51 Franklin Street, Fifth Floor
//  Boston, MA  02110-1301 USA.
//
// ---------------------------------------------------------------------

#include "config.h"

#include "main.h"
#include "flpost_ui.h"
#include "debug.h"
#include "status.h"

//#include "util.h"

#ifdef WIN32
#	include "flpostrc.h"
#	include "compat.h"
#	define dirent fl_dirent_no_thanks
#endif

#include <FL/filename.H>

#ifdef __MINGW32__
#	if LINSIM_FLTK_API_MAJOR == 1 && LINSIM_FLTK_API_MINOR < 3
#		undef dirent
#		include <dirent.h>
#	endif
#	ifndef IDI_ICON
#		define IDI_ICON 101
#	endif
#else
#	include <dirent.h>
#endif

using namespace std;

void fatal_error(string sz_error);

string BaseDir = "";
string HomeDir = "";

c_smtp_mail mail_io;

#define KNAME "flpost"
#if !defined(__WIN32__) && !defined(__APPLE__)

#	include "flpost_icon.cxx"

Pixmap	flpost_icon_pixmap;

void make_pixmap(Pixmap *xpm, const char **data)
{
	Fl_Window w(0,0, PACKAGE_NAME);
	w.xclass(PACKAGE_NAME);
	w.show();
	w.make_current();
	Fl_Pixmap icon(data);
	int maxd = (icon.w() > icon.h()) ? icon.w() : icon.h();
	*xpm = fl_create_offscreen(maxd, maxd);
	fl_begin_offscreen(*xpm);
	fl_color(FL_BACKGROUND_COLOR);
	fl_rectf(0, 0, maxd, maxd);
	icon.draw(maxd - icon.w(), maxd - icon.h());
	fl_end_offscreen();
}

#endif

Fl_Double_Window *flpost_main_window = (Fl_Double_Window *)0;
Fl_Double_Window *flpost_config_window = (Fl_Double_Window *)0;

std::string title;
char progdir[80];

int handle (int e) {
	if (Fl::focus() == flpost_main_window)
		return (e == FL_SHORTCUT);
	return 0;
}

void visit_URL(void* arg)
{
	const char* url = reinterpret_cast<const char *>(arg);
#ifndef __WOE32__
	const char* browsers[] = {
#  ifdef __APPLE__
		getenv("FLDIGI_BROWSER"), // valid for any OS - set by user
		"open"                    // OS X
#  else
		"fl-xdg-open",            // Puppy Linux
		"xdg-open",               // other Unix-Linux distros
		getenv("FLDIGI_BROWSER"), // force use of spec'd browser
		getenv("BROWSER"),        // most Linux distributions
		"sensible-browser",
		"firefox",
		"mozilla"                 // must be something out there!
#  endif
	};
	switch (fork()) {
	case 0:
#  ifndef NDEBUG
		unsetenv("MALLOC_CHECK_");
		unsetenv("MALLOC_PERTURB_");
#  endif
		for (size_t i = 0; i < sizeof(browsers)/sizeof(browsers[0]); i++)
			if (browsers[i])
				execlp(browsers[i], browsers[i], url, (char*)0);
		fatal_error("Could not find a web browser");
		exit(EXIT_FAILURE);
	case -1:
		fl_alert(
"Could not run a web browser:\n%s\n\n"
"Open this URL manually:\n%s",
			 strerror(errno), url);
	}
#else
	// gurgle... gurgle... HOWL
	// "The return value is cast as an HINSTANCE for backward
	// compatibility with 16-bit Windows applications. It is
	// not a true HINSTANCE, however. The only thing that can
	// be done with the returned HINSTANCE is to cast it to an
	// int and compare it with the value 32 or one of the error
	// codes below." (Error codes omitted to preserve sanity).
	if ((int)ShellExecute(NULL, "open", url, NULL, NULL, SW_SHOWNORMAL) <= 32)
		fl_alert("Could not open url:\n%s\n", url);
#endif
}

pthread_t *comp_thread = 0;
pthread_mutex_t mutex_comp = PTHREAD_MUTEX_INITIALIZER;
bool exit_comp_loop = false;
bool op_abort = false;

void clean_exit()
{
	progStatus.saveLastState();
	flpost_main_window->hide();
	flpost_config_window->hide();
	debug::stop();
}

void init_mail()
{
	if (progStatus.mail_from.empty()) progStatus.mail_from = FROM;
	if (progStatus.mail_out_url.empty()) progStatus.mail_out_url = SMTP_SERVER;
	if (progStatus.mail_out_port.empty()) progStatus.mail_out_port = SMTP_PORT;
	if (progStatus.mail_user_name.empty()) progStatus.mail_user_name = USERNAME;
	if (progStatus.mail_user_password.empty()) progStatus.mail_user_password = PASSWORD;

	inp_mail_out_url->value(progStatus.mail_out_url.c_str());
	inp_mail_out_port->value(progStatus.mail_out_url.c_str());
	inp_mail_user_name->value(progStatus.mail_user_name.c_str());
	inp_mail_user_password->value(progStatus.mail_user_password.c_str());

	mail_io.server_url(progStatus.mail_out_url);
	mail_io.server_port(progStatus.mail_out_port);
	mail_io.username(progStatus.mail_user_name);
	mail_io.password(progStatus.mail_user_password);

	inp_from->value(progStatus.mail_from.c_str());
	mail_io.from(progStatus.mail_from);

}

void defaults()
{
	inp_from->value(progStatus.mail_from.c_str());
	inp_to->value(TO);		mail_io.to(TO);
	inp_cc->value(CC);		mail_io.cc(CC);
	inp_subj->value(SUBJ);	mail_io.subj(SUBJ);
	inp_body->value(BODY);	mail_io.body(BODY);
}

void exit_main(Fl_Widget *w)
{
	if (Fl::event_key() == FL_Escape)
		return;
	clean_exit();
}

// Show an error dialog and print to cerr if available.
// On win32 Fl::fatal displays its own error window.
void fatal_error(string sz_error)
{
	string s = "Fatal error!\n";
	s.append(sz_error).append("\n").append(strerror(errno));

// Win32 will display a MessageBox error message
#if !defined(__WOE32__)
	fl_message_font(FL_HELVETICA, FL_NORMAL_SIZE);
	fl_alert("%s", s.c_str());
#endif
	Fl::fatal(s.c_str());
}

int main(int argc, char **argv)
{
	int arg_idx;

#ifdef __WOE32__
	char dirbuf[FL_PATH_MAX + 1];
	fl_filename_expand(dirbuf, sizeof(dirbuf) -1, "$USERPROFILE/");
	BaseDir.assign(dirbuf);
	HomeDir.assign(dirbuf).append("flpost.files/");
#else
	char dirbuf[FL_PATH_MAX + 1];
	fl_filename_expand(dirbuf, sizeof(dirbuf) -1, "$HOME/");
	BaseDir.assign(dirbuf);
	HomeDir.assign(dirbuf).append(".flpost/");
#endif

	if (mkdir(HomeDir.c_str(), 0777) == -1 && errno != EEXIST) {
		string s = "Could not create flpost directory ";
		s.append(HomeDir);
		fatal_error(s);
	}

	time_t t = time(NULL);
	string debug_file = HomeDir;
	debug_file.append("debug_log.txt");
	debug::start(debug_file.c_str());
	LOG(debug::INFO_LEVEL, debug::LOG_OTHER, "%s log started on %s", PACKAGE_STRING, ctime(&t));

	Fl::args(argc, argv, arg_idx, parse_args);

	Fl::add_handler (handle);

	flpost_main_window = flpost_main();
	flpost_config_window = flpost_config();

	progStatus.loadLastState();
	flpost_main_window->position(progStatus.mainX, progStatus.mainY);

	title.assign("flpost ").append(flpost_VERSION);

	flpost_main_window->label(title.c_str());
	flpost_main_window->callback(exit_main);

	init_mail();

	Fl::lock();
	Fl::scheme("gtk+");

#ifdef WIN32
	flpost_main_window->icon((char*)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON)));
	flpost_main_window->show(argc, argv);
#elif !defined(__APPLE__)
	make_pixmap(&flpost_icon_pixmap, flpost_icon);
	flpost_main_window->icon((char *)flpost_icon_pixmap);
	flpost_main_window->show(argc, argv);
#else
	flpost_main_window->show(argc, argv);
#endif

	return Fl::run();
}

int parse_args(int argc, char **argv, int& idx)
{
	if (strcasecmp("--help", argv[idx]) == 0) {
		printf("Usage: \n\
  --help this help text\n\
  --version\n");
		exit(0);
	}
	if (strcasecmp("--version", argv[idx]) == 0) {
		printf("Version: " VERSION "\n");
		exit (0);
	}
	return 0;
}

void close_output_dialog()
{
//	select_output_dialog->hide();
}

char szAbout[200];
void about()
{
    snprintf (szAbout, sizeof(szAbout),"\
%s\n\n\
NBEMS post office\n\
Author: W1HKJ\n\n\
Report problems to %s", PACKAGE_STRING, PACKAGE_BUGREPORT);
	fl_message("%s", szAbout);
}

void close_config()
{
	flpost_config_window->hide();
}

void flpost_configure()
{
	inp_mail_out_url->value(mail_io.server_url().c_str());
	inp_mail_out_port->value(mail_io.server_port().c_str());
	inp_mail_in_url->value(mail_io.server_url().c_str());
	inp_mail_in_port->value(mail_io.server_port().c_str());
	inp_mail_user_name->value(mail_io.username().c_str());
	inp_mail_user_password->value(mail_io.password().c_str());
	flpost_config_window->show();
}

void send()
{
	debug::show();

	if (mail_io.from().empty()) return;
	if (mail_io.to().empty() && mail_io.cc().empty()) return;
	if (mail_io.subj().empty())
		mail_io.subj("flpost mail");
	if (mail_io.body().empty())
		mail_io.body("test message");

	mail_io.send_mail();
}

void cb_message_to()
{
	mail_io.to(inp_to->value());
}

void cb_message_from()
{
	mail_io.from(inp_from->value());
}

void cb_message_cc()
{
	mail_io.cc(inp_cc->value());
}

void cb_message_subj()
{
	mail_io.subj(inp_subj->value());
}

void cb_message_body()
{
	mail_io.body(inp_body->value());
}

void cb_mail_out_url()
{
	mail_io.server_url(inp_mail_out_url->value());
}

void cb_mail_out_port()
{
	mail_io.server_port(inp_mail_out_url->value());
}

void cb_mail_in_url() {}
void cb_mail_in_port(){}

void cb_mail_user_name()
{
	mail_io.username(inp_mail_user_name->value());
}

void cb_mail_user_password()
{
	mail_io.password(inp_mail_user_password->value());
}

void set_post_master()
{
}

void post_server_name()
{
}

void clear_mail()
{
	inp_from->value(progStatus.mail_from.c_str());
	inp_to->value("");		mail_io.to("");
	inp_cc->value("");		mail_io.cc("");
	inp_subj->value("");	mail_io.subj("");
	inp_body->value("");	mail_io.body("");
}
